#include "VideoSequence.h"

VideoSequence::VideoSequence(void): QObject()
{
	IsVideoLoaded=false;
	curFrameIndex=0;
	numFrames=0;
}

VideoSequence::~VideoSequence(void)
{
}

//-------------------------------------------------------------------------------------------
// the function to copy the data from another instance
//-------------------------------------------------------------------------------------------
void VideoSequence::copyData(const VideoSequence& videoSequence)
{
	framePath=videoSequence.framePath;
	fileList=videoSequence.fileList;
	m_Image=videoSequence.m_Image;
	curFrameIndex=videoSequence.curFrameIndex;
	numFrames=videoSequence.numFrames;
}

VideoSequence::VideoSequence(const VideoSequence& videoSequence)
{
	copyData(videoSequence);
}

void VideoSequence::clear()
{
	framePath.clear();
	fileList.clear();
	numFrames=0;
	curFrameIndex=0;
}

void VideoSequence::Initialize(const QString& path,const QFileInfoList& filelist)
{
	framePath=path;
	fileList.clear();
	for(int i=0;i<filelist.size();i++)
		fileList.append(filelist[i].fileName());

	numFrames=fileList.size();
	curFrameIndex=0;
	gotoFrame(curFrameIndex+1);
	IsVideoLoaded=true;
	emit NumFramesChanged(numFrames);
}

//----------------------------------------------------------------------------------------------------------------------
// function to load the current frame according to curFrameIndex
//----------------------------------------------------------------------------------------------------------------------
bool VideoSequence::loadFrame()
{
	if(curFrameIndex<0 || curFrameIndex>=numFrames)
		return false;
	if(m_Image.load(framePath+fileList[curFrameIndex])==false)
		return false;

	emit ImageChanged(m_Image);
	return true;
}

//----------------------------------------------------------------------------------------------------------------------
// function to set the control of frame indexing
//----------------------------------------------------------------------------------------------------------------------
void VideoSequence::updateFrameControl()
{
	if(curFrameIndex==0)
		emit FrameControlChanged(false,false,true,true);
	else if(curFrameIndex==numFrames-1)
		emit FrameControlChanged(true,true,false,false);
	else
		emit FrameControlChanged(true,true,true,true);
	emit FrameIndexChanged(curFrameIndex+1);
}

//----------------------------------------------------------------------------------------------------------------------
// Goto frame findex. Note that findex ranges [1,numFrames] instead of C++ way [0,numFrames-1] so
// that other widgets can directly control change of frames. So increase the frame number by 1 before
// calling this function
//----------------------------------------------------------------------------------------------------------------------
void VideoSequence::gotoFrame(int findex)
{
	findex--;
	if(findex<0 || findex>=numFrames)
		return;
	curFrameIndex=findex;
	loadFrame();
	updateFrameControl();
	emit Update();
}

void VideoSequence::gotoFirstFrame()
{
	if(curFrameIndex==0)
		return;
	gotoFrame(1);
}

void VideoSequence::gotoPreFrame()
{
	if(curFrameIndex==0)
		return;
	gotoFrame(curFrameIndex);
}

void VideoSequence::gotoNextFrame()
{
	if(curFrameIndex==numFrames-1)
		return;
	gotoFrame(curFrameIndex+2);
}

void VideoSequence::gotoLastFrame()
{
	if(curFrameIndex==numFrames-1)
		return;
	gotoFrame(numFrames);
}

